/*
    This file is part of the KDE games library

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "kcarddialog.h"
#include "ui_kgamecardselector.h"

#include <QMatrix>
#include <QPixmap>
#include <QListWidget>
#include <QListWidgetItem>
#include <QFileInfo>
#include <QDir>

#include <klocale.h>
#include <kstandarddirs.h>
#include <krandom.h>
#include <kconfig.h>

#include <kdebug.h>

#define KCARD_DEFAULTDECK QString::fromLatin1("deck0.png")
#define KCARD_DEFAULTCARD QString::fromLatin1("11.png")
#define KCARD_DEFAULTCARDDIR QString::fromLatin1("cards-default/")

// values for the resize slider
#define SLIDER_MIN 400
#define SLIDER_MAX 3000

// KConfig entries
#define CONF_GROUP "KCardDialog"
#define CONF_RANDOMDECK QString::fromLatin1("RandomDeck")
#define CONF_DECK "Deck"
#define CONF_CARDDIR QString::fromLatin1("CardDir")
#define CONF_RANDOMCARDDIR QString::fromLatin1("RandomCardDir")
#define CONF_USEGLOBALDECK QString::fromLatin1("GlobalDeck")
#define CONF_USEGLOBALCARDDIR QString::fromLatin1("GlobalCardDir")
#define CONF_SCALE QString::fromLatin1("Scale")

#define CONF_GLOBAL_GROUP QString::fromLatin1("KCardDialog Settings")
#define CONF_GLOBAL_DECK "GlobalDeck"
#define CONF_GLOBAL_CARDDIR QString::fromLatin1("GlobalCardDir")
#define CONF_GLOBAL_RANDOMDECK QString::fromLatin1("GlobalRandomDeck")
#define CONF_GLOBAL_RANDOMCARDDIR QString::fromLatin1("GlobalRandomCardDir")


class KCardDialogPrivate
{
public:
  
   class KCardInfo
   {
     public:
       QString name;
       QString comment;
       QString path;
       QString back;
       QPixmap preview;
   };
   class KDeckInfo
   {
     public:
       QString name;
       QString comment;
       QString path;
       QPixmap preview;
   };

    KCardDialogPrivate()
    {
       randomDeck    = 0;
       randomCardDir = 0;
    }

    int randomDeck;
    int randomCardDir;

    //set query variables
    KCardDialog::CardFlags cFlags;
    QString cDeck;
    QString cCardDir;
    double cScale;

    Ui::KGameCardSelectorBase ui;
    QMap<QString, KCardInfo> cardInfo;
    QMap<QString, KDeckInfo> deckInfo;
};

int KCardDialog::getCardDeck(QString &pDeck, QString &pCardDir, QWidget *pParent,
                             CardFlags pFlags, bool* pRandomDeck, bool* pRandomCardDir,
			     double* pScale, KConfig* pConf)
{
    KCardDialog dlg(pParent, pFlags);


    dlg.setDeck(pDeck);
    dlg.setCardDir(pCardDir);

    QWidget widget(&dlg);
    dlg.d->ui.setupUi(&widget);
    dlg.setMainWidget(&widget);
    dlg.readFronts();
    dlg.readBacks();
    dlg.insertCardIcons();
    dlg.insertDeckIcons();
    connect(dlg.d->ui.frontList, SIGNAL(currentItemChanged ( QListWidgetItem * , QListWidgetItem * )),
            &dlg, SLOT(updateFront(QListWidgetItem * , QListWidgetItem * )));
    connect(dlg.d->ui.backList, SIGNAL(currentItemChanged ( QListWidgetItem * , QListWidgetItem * )),
            &dlg, SLOT(updateBack(QListWidgetItem * , QListWidgetItem * )));

    // dlg.setupDialog(pScale != 0);

    dlg.loadConfig(pConf);
    int result=dlg.exec();
    if (result==QDialog::Accepted)
    {
    // TODO check for global cards/decks!!!!
        pDeck=dlg.deck();
        pCardDir=dlg.cardDir();
        if (!pCardDir.isNull() && pCardDir.right(1)!=QString::fromLatin1("/"))
        {
            pCardDir+=QString::fromLatin1("/");
        }
        if (pRandomDeck)
        {
            *pRandomDeck = dlg.isRandomDeck();
        }
        if (pRandomCardDir)
        {
            *pRandomCardDir = dlg.isRandomCardDir();
        }
	if (pScale)
	{
            *pScale = dlg.cardScale();
	}

    }
    dlg.saveConfig(pConf);
    return result;
}

void KCardDialog::getConfigCardDeck(KConfig* conf, QString &pDeck, QString &pCardDir, double& pScale)
{
// TODO check for global cards/decks!!!!
 if (!conf) {
	return;
 }
 KConfigGroup cg(conf, CONF_GROUP);

 if (cg.readEntry(CONF_RANDOMDECK,false) || !cg.hasKey(CONF_DECK)) {
	pDeck = getRandomDeck();
 } else {
	pDeck = cg.readEntry(CONF_DECK);
 }
 if (cg.readEntry(CONF_RANDOMCARDDIR,false) || !cg.hasKey(CONF_CARDDIR)) {
	pCardDir = getRandomCardDir();
 } else {
	pCardDir = cg.readPathEntry(CONF_CARDDIR);
 }
 pScale = cg.readEntry(CONF_SCALE, 1.0);

}

QString KCardDialog::getDefaultDeck()
{
    KCardDialog::init();
    return QLatin1String("decks/") + KCARD_DEFAULTDECK;
}

QString KCardDialog::getDefaultCardDir()
{
    KCardDialog::init();

    return KCARD_DEFAULTCARDDIR;
}

QString KCardDialog::getCardPath(const QString &carddir, int index)
{
    KCardDialog::init();

    QString entry = carddir + QString::number(index) + QLatin1String(".png");
    return KStandardDirs::locate( "cards", entry );
}

QString KCardDialog::deck() const { return d->cDeck; }
void KCardDialog::setDeck(const QString& file) { d->cDeck=file; }
QString KCardDialog::cardDir() const { return d->cCardDir; }
void KCardDialog::setCardDir(const QString& dir) { d->cCardDir=dir; }
KCardDialog::CardFlags KCardDialog::flags() const { return d->cFlags; }
double KCardDialog::cardScale() const { return d->cScale; }
bool KCardDialog::isRandomDeck() const
{ return  false; }
bool KCardDialog::isRandomCardDir() const
{ return  false; }
bool KCardDialog::isGlobalDeck() const
{ return  false; }
bool KCardDialog::isGlobalCardDir() const
{ return  false; }


void KCardDialog::readFronts()
{
    // Empty data
    d->cardInfo.clear();

    QStringList svg;
    // Add SVG card sets
    svg = KGlobal::dirs()->findAllResources("cards", "svg*/index.desktop", KStandardDirs::NoDuplicates);
    QStringList list = svg+KGlobal::dirs()->findAllResources("cards", "card*/index.desktop", KStandardDirs::NoDuplicates);

    if (list.isEmpty()) return;

    for (QStringList::ConstIterator it = list.begin(); it != list.end(); ++it)
    {
        KConfig cfg(*it, KConfig::OnlyLocal);
        KConfigGroup cfgcg(&cfg, "KDE Backdeck");
        QString path = (*it).left((*it).lastIndexOf('/') + 1);
        Q_ASSERT(path[path.length() - 1] == '/');
        QPixmap pixmap(path + cfgcg.readEntry("Preview", "12c.png"));

        if (pixmap.isNull()) continue;

        QString name = cfgcg.readEntry("Name", i18n("unnamed"));
        KCardDialogPrivate::KCardInfo info;
        info.name         = name;
        info.comment      = cfgcg.readEntry("Comment",i18n("KDE card deck"));
        info.preview      = pixmap; 
        info.path         = path;
        info.back         = cfgcg.readEntry("Back",QString());
        d->cardInfo[name] = info;

    }
}


void KCardDialog::insertCardIcons()
{
    // Clear GUI
    d->ui.frontList->clear();

    QSize itemSize;

    QMapIterator<QString, KCardDialogPrivate::KCardInfo> it(d->cardInfo);
    while (it.hasNext())
    {
        it.next();
        KCardDialogPrivate::KCardInfo v = it.value();

        QPixmap previewPixmap = v.preview.scaled(QSize(32,43), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);

        QListWidgetItem *item = new QListWidgetItem(v.name, d->ui.frontList);
        item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
        item->setToolTip(v.name);
        item->setData(Qt::DecorationRole, previewPixmap);
        itemSize = itemSize.expandedTo(previewPixmap.size());
    }
    d->ui.frontList->setIconSize(itemSize);
}

void KCardDialog::updateFront(QListWidgetItem * current , QListWidgetItem * last)
{
  updateFront(current->text());
}

void KCardDialog::updateFront(QString item)
{
  kDebug() << " CHANGED FRONT" << item;
  KCardDialogPrivate::KCardInfo info = d->cardInfo[item];
  QFont font;
  font.setBold(true);
  d->ui.cardName->setText(info.name);
  d->ui.cardName->setFont(font);
  kDebug() << " back" << info.back;

  d->ui.cardDescription->setText(info.comment);
  QPixmap pixmap= info.preview;
  if (pixmap.height() > d->ui.frontPreview->height())
    pixmap = pixmap.scaledToHeight(d->ui.frontPreview->height(), Qt::SmoothTransformation);
  if (pixmap.width() > d->ui.frontPreview->width())
    pixmap = pixmap.scaledToWidth(d->ui.frontPreview->width(), Qt::SmoothTransformation);
  d->ui.frontPreview->setPixmap(pixmap);

  if (!info.back.isNull())
  {
    kDebug() << "LOCK BACK";
    updateBack(info.back);
  }
}

void KCardDialog::updateBack(QListWidgetItem * current , QListWidgetItem * last)
{
  updateBack(current->text());
}

void KCardDialog::updateBack(QString item)
{
  kDebug() << " CHANGED BACK" << item;
  KCardDialogPrivate::KDeckInfo info = d->deckInfo[item];

  QPixmap pixmap= info.preview;
  if (pixmap.height() > d->ui.backPreview->height())
    pixmap = pixmap.scaledToHeight(d->ui.backPreview->height(), Qt::SmoothTransformation);
  if (pixmap.width() > d->ui.backPreview->width())
    pixmap = pixmap.scaledToWidth(d->ui.backPreview->width(), Qt::SmoothTransformation);
  d->ui.backPreview->setPixmap(pixmap);
}

void KCardDialog::insertDeckIcons()
{
    // Clear GUI
    d->ui.backList->clear();

    QSize itemSize;
    QMapIterator<QString, KCardDialogPrivate::KDeckInfo> it(d->deckInfo);
    while (it.hasNext())
    {
        it.next();
        KCardDialogPrivate::KDeckInfo v = it.value();
        QPixmap previewPixmap = v.preview.scaled(QSize(32,43), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);

        QString name = v.name;
        QListWidgetItem *item = new QListWidgetItem(name, d->ui.backList);
        item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
        item->setToolTip(name);
        item->setData(Qt::DecorationRole, previewPixmap);
        itemSize = itemSize.expandedTo(previewPixmap.size());
    }
    d->ui.backList->setIconSize(itemSize);
}

void KCardDialog::readBacks()
{
    // Empty data
    d->deckInfo.clear();

    QStringList list = KGlobal::dirs()->findAllResources("cards", "decks/*.desktop", KStandardDirs::NoDuplicates);
    if (list.isEmpty()) return;

    for (QStringList::ConstIterator it = list.begin(); it != list.end(); ++it)
    {
        KConfig cfg(*it, KConfig::OnlyLocal);
        QPixmap pixmap(getDeckName(*it));
        if (pixmap.isNull()) continue;
        //pixmap = pixmap.scaledToWidth(72, Qt::SmoothTransformation);
        QPixmap previewPixmap = pixmap.scaled(QSize(32,43), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);

        KConfigGroup cfgcg(&cfg, "KDE Cards");
        QString name = cfgcg.readEntry("Name", i18n("unnamed"));

        KCardDialogPrivate::KDeckInfo info;
        info.name         = name;
        info.path         = getDeckName(*it);
        info.comment      = cfgcg.readEntry("Comment",i18n("KDE card deck"));
        info.preview      = pixmap; 
        d->deckInfo[name] = info;
    }
}


KCardDialog::~KCardDialog()
{
 delete d;
}


// Create the dialog
KCardDialog::KCardDialog( QWidget *parent, CardFlags mFlags)
    : KDialog( parent ),
    d( new KCardDialogPrivate )
{
	setCaption(i18n("Carddeck Selection"));
	setButtons(Ok|Cancel);
	setDefaultButton(Ok);
	setModal(true);
	showButtonSeparator(true);
        KCardDialog::init();
        d->cFlags = mFlags;
        adjustSize();
}

QString KCardDialog::getDeckName(const QString &desktop)
{
    QString entry = desktop.left(desktop.length() - strlen(".desktop"));
    if (KStandardDirs::exists(entry + QString::fromLatin1(".png")))
        return entry + QString::fromLatin1(".png");

    // rather theoretical
    if (KStandardDirs::exists(entry + QString::fromLatin1(".xpm")))
        return entry + QString::fromLatin1(".xpm");
    return QString();
}

QString KCardDialog::getRandomDeck()
{
    KCardDialog::init();

    QStringList list = KGlobal::dirs()->findAllResources("cards", "decks/*.desktop");
    if (list.isEmpty())
        return QString();

    int d = KRandom::random() % list.count();
    return getDeckName(list.at(d));
}

QString KCardDialog::getRandomCardDir()
{
    KCardDialog::init();

    QStringList list = KGlobal::dirs()->findAllResources("cards", "card*/index.desktop");
    if (list.isEmpty())
        return QString();

    int d = KRandom::random() % list.count();
    QString entry = list.at(d);
    return entry.left(entry.length() - strlen("index.desktop"));
}



void KCardDialog::loadConfig(KConfig* conf)
{
 if (!conf) {
	return;
 }

 KConfigGroup cg(conf, CONF_GROUP);

 /*
 if (! (flags() & NoDeck)) {
	if (cg.hasKey(CONF_DECK)) {
		setDeck(cg.readEntry(CONF_DECK));
	}

	bool random = cg.readEntry(CONF_RANDOMDECK, false);
	d->randomDeck->setChecked(random);

	if (cg.hasKey(CONF_USEGLOBALDECK) && cg.readEntry(CONF_USEGLOBALDECK,false)) {
		d->globalDeck->setChecked(true);
	} else {
		d->globalDeck->setChecked(false);
	}
 }
 if (! (flags() & NoCards)) {
	if (cg.hasKey(CONF_CARDDIR)) {
		setCardDir(cg.readPathEntry(CONF_CARDDIR));
	}

	bool random = cg.readEntry(CONF_RANDOMCARDDIR, false);
	d->randomCardDir->setChecked(random);

	if (cg.hasKey(CONF_USEGLOBALCARDDIR) && cg.readEntry(CONF_USEGLOBALCARDDIR,false)) {
		d->globalCardDir->setChecked(true);
	} else {
		d->globalCardDir->setChecked(false);
	}
 }

 d->cScale = cg.readEntry(CONF_SCALE, 1.0);
 */
}


void KCardDialog::saveConfig(KConfig* conf)
{
 if (!conf) {
	return;
 }
 /*
 KConfigGroup cg(conf, CONF_GROUP);
 if (! (flags() & NoDeck)) {
	cg.writeEntry(CONF_DECK, deck());
	cg.writeEntry(CONF_RANDOMDECK, isRandomDeck());
	cg.writeEntry(CONF_USEGLOBALDECK, d->globalDeck->isChecked());
 }
 if (! (flags() & NoCards)) {
	cg.writePathEntry(CONF_CARDDIR, cardDir());
	cg.writeEntry(CONF_RANDOMCARDDIR, isRandomCardDir());
	cg.writeEntry(CONF_USEGLOBALCARDDIR, d->globalCardDir->isChecked());
 }
 cg.writeEntry(CONF_SCALE, d->cScale);
 */
}


void KCardDialog::init()
{
    static bool _inited = false;
    if (_inited) return;
    KGlobal::dirs()->addResourceType("cards", "data", "carddecks/");
    KGlobal::locale()->insertCatalog("libkdegames");
    _inited = true;
}


// Check whether the card back deck contains also an SVG file.
bool KCardDialog::isSVGDeck(const QString& deck)
{
  return !deckSVGFilePath(deck).isNull();
}

// Check whether the card set is SVG or not.
bool KCardDialog::isSVGCards(const QString& cardDir)
{
  return !cardSVGFilePath(cardDir).isNull();
}


// Retreive the SVG file belonging to the given card back deck.
QString KCardDialog::deckSVGFilePath(const QString& deck)
{
  QFileInfo info(deck);
  QString deckIndex = deck;
  deckIndex.replace(".png",".desktop");

  // Card deck section
  KConfig deckInfo( deckIndex, KConfig::OnlyLocal);
  KConfigGroup deckGroup(&deckInfo, "KDE Cards");
  QString deckSVG  = deckGroup.readEntry("SVG", QString());
  if (deckSVG.isNull()) return deckSVG;

  QFileInfo svgInfo(QDir(info.path()), deckSVG);
  return svgInfo.filePath();
}


// Retreive the SVG file belonging to the given card back deck.
QString KCardDialog::cardSVGFilePath(const QString& cardDir)
{
  KConfig cardInfo(cardDir+"/index.desktop", KConfig::OnlyLocal);
  KConfigGroup cardGroup(&cardInfo, "KDE Backdeck");
  QString cardSVG    = cardGroup.readEntry("SVG", QString());
  if (cardSVG.isNull()) return cardSVG;
  QFileInfo svgInfo(QDir(cardDir), cardSVG);
  return svgInfo.filePath();
}


#include "kcarddialog.moc"
